﻿using System;
using System.Collections.ObjectModel;
using BuiltToRoam.Hosting;

namespace JavascriptHost
{
    public sealed partial class MainPage 
    {
        public MainPage()
        {
            DataContext = new MainViewModel();
            InitializeComponent();
        }
        public MainViewModel CurrentViewModel => DataContext as MainViewModel;
    }

    public class MainViewModel
    {
        public string CodeToExecute { get; set; }

        public ObservableCollection<string> Output { get; } = new ObservableCollection<string>();

        private JavaScriptRuntime Runtime { get; }

        private JavaScriptNativeFunction EchoDelegate { get; set; }

        public MainViewModel()
        {
            Runtime = JavaScriptRuntime.Create();
        }

        public void Execute()
        {
            using (var context = Runtime.CreateContext())
            using (new JavaScriptContext.Scope(context))
            {
                try
                {
                    JavaScriptContext.StartDebugging();

                    DefineEcho();

                    GC.Collect();

                    var result = JavaScriptContext.RunScript(CodeToExecute);

                    var numberResult = result.ConvertToNumber();
                    var doubleResult = numberResult.ToDouble();
                    Output.Add("Result: " + doubleResult);

                }
                catch (Exception ex)
                {
                    Output.Add("Exception:" + ex.Message);
                }

            }
        }


        private void DefineEcho()
        {
            var globalObject = JavaScriptValue.GlobalObject;

            var hostObject = JavaScriptValue.CreateObject();
            var hostPropertyId = JavaScriptPropertyId.FromString("managedhost");
            globalObject.SetProperty(hostPropertyId, hostObject, true);
            EchoDelegate = Echo;
            DefineCallback(hostObject, "echo", EchoDelegate);
        }


        private void DefineCallback(JavaScriptValue hostObject, string callbackName, JavaScriptNativeFunction callbackDelegate)
        {
            var propertyId = JavaScriptPropertyId.FromString(callbackName);

            var function = JavaScriptValue.CreateFunction(callbackDelegate);

            hostObject.SetProperty(propertyId, function, true);
        }

        private JavaScriptValue Echo(JavaScriptValue callee, bool isConstructCall, JavaScriptValue[] arguments,
            ushort argumentCount, IntPtr callbackData)
        {
            for (uint index = 1; index < argumentCount; index++)
            {
                Output.Add(arguments[index].ConvertToString().ToString());
            }

            return JavaScriptValue.True;
        }
    }
}
